use anyhow::Error;

use super::Repository;
use crate::database::Queryer;
use crate::domain::rolle::entity;
use crate::scalar::Id;

impl Repository {
    pub async fn create_rolle<'c, C: Queryer<'c>>(
        &self,
        db: C,
        rolle: &entity::Rolle,
    ) -> Result<Id, Error> {
        const QUERY: &str = "insert into rolle (id, created_at, updated_at,
                              name) values ($1, $2, $3, $4) returning id";

        match sqlx::query_scalar::<_, Id>(QUERY)
            .bind(rolle.id)
            .bind(rolle.created_at)
            .bind(rolle.updated_at)
            .bind(&rolle.rollenname)
            .fetch_one(db)
            .await
        {
            Err(err) => {
                tracing::error!("{}", &err);
                Err(err.into())
            }
            Ok(id) => Ok(id),
        }
    }
}
