use anyhow::Error;

use super::Repository;
use crate::database::Queryer;
use crate::domain::rolle::model::{self};
use crate::scalar::Id;

impl Repository {
    pub async fn find_rolle_by_id<'c, C: Queryer<'c>>(
        &self,
        db: C,
        id: Id,
    ) -> Result<model::Rolle, Error> {
        const QUERY: &str = r#"
             select
               r.id, r.created_at, r.updated_at, r.rollenname,
              (g.id, g.created_at, g.updated_at, g.gruppenname) as "gruppen!: Vec<model::Gruppe>"
             from rollen as r
             join rollen_gruppen as rg on rg.rolle_id = r.id
             join gruppen as g on rg.gruppe_id = g.id
             where r.id = $1;
         "#;

        let row = sqlx::query_as::<_, model::Rolle>(QUERY)
            .bind(id)
            .fetch_one(db)
            .await?;

        // let row = sqlx::query_as!(
        //     model::Rolle,
        //     r#"
        //      select
        //        r.id, r.created_at, r.updated_at, r.rollenname,
        //       (g.id, g.created_at, g.updated_at, g.gruppenname) as "gruppen!: Vec<model::Gruppe>"
        //      from rollen as r
        //      join rollen_gruppen as rg on rg.rolle_id = r.id
        //      join gruppen as g on rg.gruppe_id = g.id
        //      where r.id = $1;
        //  "#,
        //     id
        // )
        // .fetch_one(db)
        // .await?;

        Ok(row)
    }
}
