use async_graphql::{Context, FieldResult, Object};
use sqlx::PgPool;

use crate::{models::gruppe::Gruppe, scalar::Id};

use super::rolle::Rolle;

pub struct Benutzer {
    pub id: Id,
    pub kennung: String,
    pub vorname: String,
    pub nachname: String,
}

#[Object]
impl Benutzer {
    pub async fn id(&self) -> Id {
        self.id
    }

    pub async fn kennung(&self) -> &str {
        &self.kennung
    }

    pub async fn vorname(&self) -> &str {
        &self.vorname
    }

    pub async fn nachname(&self) -> &str {
        &self.nachname
    }

    pub async fn rollen<'ctx>(&self, ctx: &Context<'ctx>) -> FieldResult<Vec<Rolle>> {
        let pool = ctx.data::<PgPool>()?;

        let rows = sqlx::query_as!(
            Rolle,
            r#"
                SELECT
                    r.id,
                    r.rollenname,
                    r.erstellt_am,
                    r.geaendert_am
                FROM rollen AS r
                LEFT JOIN benutzer_rollen AS br ON r.id = br.rolle_id
                WHERE br.benutzer_id = $1;
            "#,
            &self.id
        )
        .fetch_all(pool)
        .await?;

        Ok(rows)
    }

    pub async fn gruppen<'ctx>(&self, ctx: &Context<'ctx>) -> FieldResult<Vec<Gruppe>> {
        let pool = ctx.data::<PgPool>()?;

        let rows = sqlx::query_as!(
            Gruppe,
            r#"
                SELECT
                    g.id,
                    g.gruppenname,
                    g.erstellt_am,
                    g.geaendert_am
                FROM gruppen AS g
                LEFT JOIN benutzer_gruppen AS bg ON g.id = bg.gruppe_id
                WHERE bg.benutzer_id = $1;
            "#,
            &self.id
        )
        .fetch_all(pool)
        .await?;

        Ok(rows)
    }
}
