use async_graphql::{Context, FieldResult, Object};
use sqlx::PgPool;

use crate::scalar::{Id, Time};

use super::gruppe::Gruppe;

#[derive(Debug)]
pub struct Rolle {
    pub id: Id,
    pub rollenname: String,

    pub erstellt_am: Time,
    pub geaendert_am: Time,
}

#[Object]
impl Rolle {
    pub async fn id(&self) -> Id {
        self.id
    }

    pub async fn rollename(&self) -> &str {
        &self.rollenname
    }

    pub async fn erstellt_am(&self) -> Time {
        self.erstellt_am
    }

    pub async fn geaendert_am(&self) -> Time {
        self.geaendert_am
    }

    pub async fn gruppen<'ctx>(&self, ctx: &Context<'ctx>) -> FieldResult<Vec<Gruppe>> {
        let pool = ctx.data::<PgPool>()?;

        let rows = sqlx::query_as!(
            Gruppe,
            r#"
                SELECT
                    g.id,
                    g.gruppenname,
                    g.erstellt_am,
                    g.geaendert_am
                FROM gruppen AS g
                LEFT JOIN rollen_gruppen AS rg ON g.id = rg.gruppe_id
                WHERE rg.rolle_id = $1;
            "#,
            &self.id
        )
        .fetch_all(pool)
        .await?;

        Ok(rows)
    }
}
