use crate::{
    models::hersteller::{Hersteller, HerstellerCreateInput, HerstellerUpdateInput},
    scalar::Id,
};
use async_graphql::{Context, FieldResult};
use sqlx::postgres::PgPool;

#[derive(Default)]
pub struct HerstellerMutation;

#[async_graphql::Object]
impl HerstellerMutation {
    async fn create_hersteller(
        &self,
        ctx: &Context<'_>,
        input: HerstellerCreateInput,
    ) -> FieldResult<Hersteller> {
        let pool = ctx.data::<PgPool>()?;
        let row = Hersteller::create(pool, &input).await?;
        Ok(row)
    }

    async fn update_hersteller(
        &self,
        ctx: &Context<'_>,
        input: HerstellerUpdateInput,
    ) -> FieldResult<Hersteller> {
        let pool = ctx.data::<PgPool>()?;
        let row = Hersteller::update(pool, &input).await?;
        Ok(row)
    }

    async fn delete_hersteller(&self, ctx: &Context<'_>, id: Id) -> FieldResult<bool> {
        let pool = ctx.data::<PgPool>()?;
        Ok(Hersteller::delete(pool, &id).await?)
    }
}
