use crate::{
    models::typ::{Typ, TypCreateInput, TypUpdateInput},
    scalar::Id,
};
use async_graphql::{Context, FieldResult};
use sqlx::postgres::PgPool;

#[derive(Default)]
pub struct TypMutation;

#[async_graphql::Object]
impl TypMutation {
    async fn create_typ(&self, ctx: &Context<'_>, input: TypCreateInput) -> FieldResult<Typ> {
        let pool = ctx.data::<PgPool>()?;

        let row = Typ::create(pool, &input).await?;
        Ok(row)
    }

    async fn create_many_typen(
        &self,
        ctx: &Context<'_>,
        input: Vec<TypCreateInput>,
    ) -> FieldResult<Vec<Typ>> {
        let pool = ctx.data::<PgPool>()?;
        let row = Typ::create_many(pool, &input).await?;
        Ok(row)
    }

    async fn update_typ(&self, ctx: &Context<'_>, input: TypUpdateInput) -> FieldResult<Typ> {
        let pool = ctx.data::<PgPool>()?;
        let row = Typ::update(pool, &input).await?;
        Ok(row)
    }

    async fn delete_typ(&self, ctx: &Context<'_>, id: Id) -> FieldResult<bool> {
        let pool = ctx.data::<PgPool>()?;
        Ok(Typ::delete(pool, &id).await?)
    }
}
